#
# Copyright (C) 2009 OpenWrt.org
# Frederic Pecourt 2011
#
. /lib/upgrade/gemini.sh

USE_REFRESH=1

CI_BLKSZ=65536
CI_LDADR=0x80060000

v() {
	[ "$VERBOSE" -ge 1 ] && echo "$@"
}

platform_find_partitions() {
	local first dev size erasesize name
	while read dev size erasesize name; do
		name=${name#'"'}; name=${name%'"'}
		case "$name" in
			Kern|Ramdisk)
				if [ -z "$first" ]; then
					first="$name"
				else
					echo "$erasesize:$first:$name"
					break
				fi
			;;
		esac
	done < /proc/mtd
}

platform_find_kernelpart() {
	local part
	for part in "${1%:*}" "${1#*:}"; do
		case "$part" in
			Kern)
				echo "$part"
				break
			;;
		esac
	done
}

platform_do_upgrade_combined() {
	v "Platform system combined upgrade..."
	sync
	local partitions=$(platform_find_partitions)
	local kernelpart=$(platform_find_kernelpart "${partitions#*:}")
	local erase_size=$((0x${partitions%%:*})); partitions="${partitions#*:}"
	local kern_length=0x$(dd if="$1" bs=2 skip=2 count=4 2>/dev/null)
	local kern_blocks=$(($kern_length / $CI_BLKSZ))
	local root_blocks=$((0x$(dd if="$1" bs=2 skip=6 count=4 2>/dev/null) / $CI_BLKSZ))

	if [ -n "$partitions" ] && [ -n "$kernelpart" ] && \
	   [ ${kern_blocks:-0} -gt 0 ] && \
	   [ ${root_blocks:-0} -gt ${kern_blocks:-0} ] && \
	   [ ${erase_size:-0} -gt 0 ];
	then
		local append=""
		[ -f "$CONF_TAR" -a "$SAVE_CONFIG" -eq 1 ] && append="-j $CONF_TAR"
		v "Flashing $kernelpart"
		( dd if="$1" bs=$CI_BLKSZ skip=1 count=$kern_blocks 2>/dev/null; ) | \
			mtd write - $kernelpart
		v "Flashing rootfs"
		( dd if="$1" bs=$CI_BLKSZ skip=$((1+$kern_blocks)) count=$root_blocks 2>/dev/null ) | \
			mtd $append write - Ramdisk
	fi
}
platform_do_upgrade_combined_16mb() {
	v "Platform system combined upgrade..."
	tar -C /tmp -xzf $1
	rm $1
	dd if=/tmp/rd.gz of=/tmp/rootfs bs=6M count=1 >/dev/null 2>&1
	rm /tmp/rd.gz
	dd if=/tmp/hddapp.tgz of=/tmp/rootfs bs=6M count=1 seek=1 >/dev/null 2>&1
	rm /tmp/hddapp.tgz

	local line

	line=`grep Distribution /tmp/ImageInfo`
	eval $line
	line=`grep Layout /tmp/ImageInfo`
	eval $line

	if [ "$Distribution" == "OpenWrt" ]; then
		USE_REFRESH=1
	fi

	local mode=0
	if [ "$Layout" == "Compact" ]; then
		mode=1
	fi 

	v "Flashing $kernelpart"
	mtd -q write /tmp/zImage Kern
	mtd refresh Kern
	v "Flashing rootfs"
	mtd -q write /tmp/rootfs Ramdisk
}

platform_check_image() {
	local board=$(gemini_board_name)
	local magic="$(get_magic_long "$1")"

	[ "$ARGC" -gt 1 ] && return 1

	case "$board" in
	mzknas  )
		[ "$magic" != "43494d5a" ] && {
			echo "Invalid image. Use mzknas*-sysupgrade*.bin files on this board"
			return 1
		}
		;;
	mvixbox )
		[ "$magic" != "43494d56" ] && {
			echo "Invalid image. Use mvixbox*-sysupgrade*.bin files on this board"
			return 1
		}
		;;
	nas4220 )
		[ "$magic" != "43494E41" ] && {
			echo "Invalid image. Use nas4220*-sysupgrade*.bin files on this board"
			return 1
		}
		;;
	*)
		echo "Sysupgrade is not yet supported on $board."
		return 1
		;;
	esac

	local md5_img=$(dd if="$1" bs=2 skip=10 count=16 2>/dev/null)
	local md5_chk=$(dd if="$1" bs=$CI_BLKSZ skip=1 2>/dev/null | md5sum -); md5_chk="${md5_chk%% *}"

	if [ -n "$md5_img" -a -n "$md5_chk" ] && [ "$md5_img" = "$md5_chk" ]; then
		return 0
	else
		echo "Invalid image. Contents do not match checksum (image:$md5_img calculated:$md5_chk)"
		return 1
	fi
}

platform_check_image0() {
	local productline=`tar -xzOf $1 ImageInfo | grep productName`
	[ "$productline" == "" ] && return 1

	eval $productline
	[ "$productName" == "" ] && return 1

	local machine=$(awk 'BEGIN{FS="[ \t]+:[ \t]"} /Hardware/ {print $2}' /proc/cpuinfo)

	[ "$productName" != "$machine" ] && return 1

	echo $productName
	return 0
}

platform_do_upgrade() {
	local board=$(gemini_board_name)
	v "Platform system upgrade..."
	v "Found board $board"
	case "$board" in
	mzknas | mvixbox )
		platform_do_upgrade_combined "$ARGV"
		;;
	nas4220 )
		platform_do_upgrade_combined_16mb "$ARGV"
		;;
	*)
		default_do_upgrade "$ARGV"
		;;
	esac
}

disable_watchdog() {
	killall watchdog
	( ps | grep -v 'grep' | grep '/dev/watchdog' ) && {
		echo 'Could not disable watchdog'
		return 1
	}
}

append sysupgrade_pre_upgrade disable_watchdog
